import os
import sys
import importlib
from salal.core.log import log
from salal.core.config import config

class Handlers:

    #---------------------------------------------------------------------------
    @classmethod
    def load_module_source (cls, name, path):
        spec = importlib.util.spec_from_file_location(name, path)
        module = importlib.util.module_from_spec(spec)
        sys.modules[name] = module
        spec.loader.exec_module(module)
        return module
    
    #---------------------------------------------------------------------------

    @classmethod
    def load_handlers (cls, directory):
        # Handlers are a way to determine what processing to carry out
        # in a particular instance based on a 'tag' value. To
        # implement a set of handlers, in <directory> have separate
        # subdirectories for each handler. Each subdirectory should
        # contain a <handler.py> file, which should create an object
        # called <handler>.  The handler object should have a method
        # <get_tags> that returns a list of the tags that should be
        # associated with this particular handler. This method will
        # real all the <handler.py> files, and return a dict where the
        # keys are all the tags, and the values are the corresponding
        # handler objects. Generally each handler object should have
        # one or more additional methods that carry out the actual
        # processing, but what those are and how they are called is up
        # to the code that calls the <load_handler> method.

        handlers = dict()
        for extension_dir in config.system['paths']['extension_dirs']:
            if os.path.isdir(os.path.join(extension_dir, directory)):
                with os.scandir(os.path.join(extension_dir, directory)) as entries:
                    for entry in entries:
                        if entry.is_dir() and not entry.name.startswith('__'):
                            handler_relative_path = os.path.join(directory, entry.name, 'handler.py')
                            handler_full_path = os.path.join(extension_dir, handler_relative_path)
                            if not os.path.exists(handler_full_path):
                                log.message('WARN', 'Handler directory ' + entry.name + ' does not contain a handler.py file')
                            else:
                                package_specifier = os.path.normpath(handler_relative_path).replace(os.sep, '.').replace('.py', '')
                                log.message('TRACE', 'Loading handler from ' + package_specifier)
                                sys.path.insert(0, extension_dir)
                                handler_module = importlib.import_module(package_specifier)
                                sys.path.pop(0)
                                for tag in handler_module.handler.get_tags():
                                    log.message('TRACE', tag)
                                    handlers[tag] = handler_module.handler
        return handlers
    
    #---------------------------------------------------------------------------

handlers = Handlers



import os
import importlib
from salal.core.log import log
from salal.core.config import config
from salal.core.handlers import handlers
from salal.core.utilities import utilities

class Actions:

    #---------------------------------------------------------------------------

    @classmethod
    def initialize (cls):
        if 'action_commands' not in config.system:
            log.message('ERROR', 'No actions are configured')
        log.message('DEBUG', 'Loading command handlers')
        cls.handlers = handlers.load_handlers(config.system['paths']['command_handlers_dir'])

    #---------------------------------------------------------------------------

    @classmethod
    def execute_internal_command (cls, command):
        if command in cls.handlers:
            cls.handlers[command].execute(command)
        else:
            log.message('ERROR', 'Command ' + command + ' is not configured.')
        
    #---------------------------------------------------------------------------

    @classmethod
    def execute (cls, action):
        # Make sure this action is defined
        if action not in config.system['action_commands']:
            log.message('ERROR', 'The action ' + action + ' is not configured')
        else:
            log.message('INFO', 'Executing ' + action + ' action')
            
        # Iterates through the list of commands associated with 'tag',
        # does substitution for system variables, and passes them to
        # the OS for execution
        for command_spec in config.system['action_commands'][action]:
            if command_spec['type'] == 'internal':
                cls.execute_internal_command(command_spec['command'])
            elif command_spec['type'] == 'external':
                command_string = utilities.substitute_variables(command_spec['command'], config.system)
                log.message('INFO', command_string)
                os.system(command_string)
            else:
                log.message('ERROR', 'Unrecognized command type ' + command_spec['type'])
 
    #---------------------------------------------------------------------------
    
actions = Actions



import setuptools
import glob

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

additional_files = list(map(lambda entry: entry.replace('salal/', ''), glob.glob('salal/extensions/**/*.py', recursive=True)))
additional_files.append('system.json')
print(additional_files)
    
setuptools.setup(
    name="salal",
    version="0.19.0-beta",
    author="Todd Haskell",
    author_email="todd@craggypeak.com",
    description="A system for building websites from templates and content",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/haskelt/salal",
    license="GNU General Public License v3",
    packages=setuptools.find_packages(),
    package_data={'': additional_files},
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
        "Intended Audience :: Developers",
        "Topic :: Text Processing :: Markup :: HTML"
    ],
    python_requires='>=3.6',
    install_requires=[
        'Jinja2>=2.11.2'
    ]
)
